//  ---------------------------------------------------------------------------
//  This file is part of reSID, a MOS6581 SID emulator engine.
//  Copyright (C) 2004  Dag Lem <resid@nimrod.no>
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  ---------------------------------------------------------------------------

#ifndef VICE__SID_FP_H__
#define VICE__SID_FP_H__

#include "siddefs-fp.h"
#include "wave_c.h"
#include "envelope_c.h"
#include "filter_c.h"

// ----------------------------------------------------------------------------
// Amplitude modulated waveform output.
// Ideal range [-2048*255, 2047*255].
// ----------------------------------------------------------------------------
/*inline float VoiceFP::output(WaveformGeneratorFP& source)
{
    float envelope_counter_dac = env_dac[envelope.readENV()];
    return wave.output(source) * envelope_counter_dac;
}*/

typedef struct SIDFPState
{
    reg8 sid_register[0x20];
    reg8 bus_value;
    cycle_count bus_value_ttl;
} SIDFPState;

typedef struct SIDFP
{
  WaveformGeneratorFP wave[3];
  EnvFP    envelope[3];
  FilterFP filter;

  // Multiplying D/A DC offset.
  float voice_DC;
  reg8 bus_value;
  cycle_count bus_value_ttl;

  // External audio input.
  float ext_in;

  // Sampling variables.
  float cycles_per_sample;
  float sample_offset;
  int sample_index;
  
  /* for linear interpolation mode */
  float Vo, sample_prev;

  chip_model model;
} SIDFP;

#ifdef __cplusplus
extern "C" {
#endif

void sid_init(SIDFP *sid);
void sid_free(SIDFP *sid);
  
void sid_set_chip_model(SIDFP *sid, chip_model model);
void sid_set_voice_nonlinearity(SIDFP *sid, float nonlinearity);
void sid_set_sampling_parameters(SIDFP *sid, float clock_freq, sampling_method method, float sample_freq);
void sid_reset(SIDFP *sid);
FilterFP *sid_get_filter(SIDFP *sid) { return &sid->filter; }
void sid_enable_filter(SIDFP *sid, reg8 enable);
// 16-bit input (EXT IN).
void sid_input(SIDFP *sid, int sample);
// 16-bit output (AUDIO OUT)
float sid_output(SIDFP *sid);
// Read/write registers.
void sid_mute(SIDFP *sid, reg8 channel, reg8 enable);
reg8 sid_read(SIDFP *sid, reg8 offset);
void sid_write(SIDFP *sid, reg8 offset, reg8 value);
// Clock SID & output samples.
float sid_clock1(SIDFP *sid);
//void sid_clock_n(SIDFP *sid, cycle_count delta_t);
int sid_clock(SIDFP *sid, cycle_count *delta_t, float* buf, unsigned int n);

// Read/write state.
void sid_read_state(SIDFP *sid, SIDFPState *state);
void sid_write_state(SIDFP *sid, const SIDFPState *state);

#ifdef __cplusplus
};
#endif

#endif // not __SID_H__

